//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////


#ifndef _DATASET
#define _DATASET

#include <iostream>
#include "example.h"

using namespace std;


/********************************************************************************/
/*                                                                              */
/*  dataset:  dataset of binary examples                                      */
/*                                                                              */
/********************************************************************************/

class dataset 
{
 private:
  
  struct mlDatasetNode {
    example* ex;          // pointer to the example
    mlDatasetNode* next;    // next example, globally
    
    mlDatasetNode() : 
      ex(NULL), next(NULL) 
    {}
    mlDatasetNode(example* e) 
      : ex(e), next(NULL)
    {}
  };
  
    
  mlDatasetNode* first;
  mlDatasetNode* last;
  int  _size;
  int *_sizes;
  int  _dimension;
  int  _nlabels;

public:

  class iterator {
    private :
      mlDatasetNode* n;

    public:
    iterator() 
      : n(NULL)
      {}

    iterator(mlDatasetNode *n0) 
      : n(n0)
      {}

    // ~iterator() { cerr << "dataset iterator destroyed\n"; }
    
    example* operator->() const { return n->ex; }

    example& operator*() const { return *(n->ex); }
    
    iterator& operator++() { n = n->next; return *this; }
    iterator operator++(int) { iterator tmp(this->n); n = n->next; return tmp; }
    
    bool operator==(const iterator& rhs) { return n == rhs.n; }
    bool operator!=(const iterator& rhs) { return n != rhs.n; }
    
  };


  dataset(int nlabels);
  ~dataset();

  void delete_examples();


  // input
  void read_stream(istream&);
  void add_example(example*, mlDatasetNode* = NULL); 

  // consultores
  int size() const { return _size; }
  int negative_size(int l) const { return _sizes[2*l]; }
  int positive_size(int l) const { return _sizes[2*l+1]; }
  int nlabels() const { return _nlabels; }
  int dimension() const { return _dimension; }
  
  // recorregut
  iterator begin() const { iterator p(first);  return p; }
  iterator end() const { iterator p(NULL); return p; }
  
  void     split(int feature, dataset *ds0, dataset *ds1, bool create_ds_nodes);
  
  void print(ostream&) const;
  void print_sizes(ostream&) const;

};

#endif



















