//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////

#ifndef _MACO
#define _MACO

#include "maco_options.h"
#include "language.h"
#include "locutions.h"
#include "dictionary.h"
#include "numbers.h"
#include "dates.h"
#include "quantities.h"
#include "punts.h"
#include "probabilities.h"
#include "np.h"

using namespace std;

////////////////////////////////////////////////////////////////
///  Class maco implements the morphological analyzer, which
/// uses all the specific analyzers: dates, numbers, dictionary, etc.
////////////////////////////////////////////////////////////////

class maco {
   private:
      /// creation options
      maco_options defaultOpt; 

      locutions loc;     
      dictionary dico;
      numbers numb;
      dates date;
      quantities quant;
      punts punt;
      probabilities prob;
      np npm;
      
   public:
      /// Constructor
      maco(const maco_options &); 

      /// analyze given sentences
      void analyze(list<sentence> &);
      /// analyze sentences, return analyzed copy
      list<sentence> analyze(const list<sentence> &);
};

#endif

