//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////

#ifndef _PROBABILITIES
#define _PROBABILITIES

#include <map>

#include "maco_options.h"
#include "language.h"

using namespace std;

////////////////////////////////////////////////////////////////
/// Class probabilities sets lexical probabilities for
/// each PoS tag of each word in a sentence.
////////////////////////////////////////////////////////////////

class probabilities {
   private:
      // Probability threshold for unknown words tags
      double MACO_ProbabilityThreshold;
      string Language;

      /// unigram probabilities
      map<string,double> single_tags;
      /// probabilities for usual ambiguity classes
      map<string,map<string,double> > class_tags;
      /// lexical probabilities for frequent words 
      map<string,map<string,double> > lexical_tags;
      /// list of tags and probabilities to assign to unknown words
      map<string,double> unk_tags;
      /// list of tag frequencies for unknown word suffixes
      map<string,map<string,double> > unk_suffs;
      /// unknown words suffix smoothing parameter;
      double theeta;
      /// length of longest suffix
      unsigned int long_suff;

      /// Compute p(tag|suffix) using recursively shorter suffixes.
      double compute_probability(const string &, double, string);

   public:
      /// Constructor
      probabilities(const maco_options &);

      /// Assign probabilities to tags using default options
      void annotate(sentence &);

};

#endif
