//////////////////////////////////////////////////////////////////
//
//    FreeLing - Open Source Language Analyzers
//
//    Copyright (C) 2004   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx C6.212 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////

#ifndef _QUANTITIES_MOD
#define _QUANTITIES_MOD

#include <map>

#include "automat.h"
#include "language.h"

////////////////////////////////////////////////////////////////
///  The abstract class quantities_module generalizes a
/// percentage, ratios, and currency expression recognizer 
/// for different languages.
////////////////////////////////////////////////////////////////

class quantities_module: public automat {

   protected:
      /// translate particular strings to token codes
      map<string,int> tok;
      /// translate fraction strings to their nummerical values
      map<string,long double> fract;
      /// list of currency names and their international codes
      map<string,string> currency;
      /// list of measure units and their lexical realizations
      map<string,string> units;

      /// values for ratios
      long double value1, value2;
      /// auxiliary for nation-specific currencies
      string currCodes;
      /// auxiliary for storing standarized unit description
      string unitCode;

   public:
      /// Constructor
      quantities_module();
};


////////////////////////////////////////////////////////////////
///   The derived class quantities_default implements a default
///   quantities recognizer (only percentages are recognized).
////////////////////////////////////////////////////////////////

class quantities_default: public quantities_module {

   private:
      int ComputeToken(int,sentence::const_iterator,const sentence &);
      void ResetActions();
      void StateActions(int, int, int, sentence::const_iterator);
      void SetMultiwordAnalysis(sentence::iterator) const;

   public:
      /// Constructor
      quantities_default(const string &);
};


////////////////////////////////////////////////////////////////
///   The derived class quantities_es implements a Spanish
///  (and Catalan) quantities recognizer.
////////////////////////////////////////////////////////////////

class quantities_es: public quantities_module {

   private:
      int ComputeToken(int,sentence::const_iterator,const sentence &);
      void ResetActions();
      void StateActions(int, int, int, sentence::const_iterator);
      void SetMultiwordAnalysis(sentence::iterator) const;

   public:
      /// Constructor
      quantities_es(const string &); 
};


#endif

